function [sp_data] = extractSuperpixelNeighborhood(segs,pixel_neighbors,r,get_neighbors)
if ~exist('pixel_neighbors','var')
    pixel_neighbors = 1;
end

if ~exist('get_neighbors','var')
    get_neighbors = 1;
end

if ~exist('r','var')
    r=1;
end

nsegs = max(segs(:));

% indices of pixels within each segment i
idx = cell(nsegs,1);

% coordiantes of pixels within each segment i
ys = cell(nsegs,1); xs = cell(nsegs,1);

% neighbor segment indices for each segment i
neighbors = cell(nsegs,1);

% pixel indices of shared edge of j-th neighbor with segment i
neighbors_edge_idx = cell(nsegs,nsegs);
neighbors_edge_xs = cell(nsegs,nsegs);
neighbors_edge_ys = cell(nsegs,nsegs);

s = strel('diamond',r);

for i=1:nsegs
    % find pixels for segment i
    mask = zeros(size(segs));
    idx{i} = find(segs==i);
    mask(idx{i}) = 1;

    % get x and y coords for pixels in segment i
    [ys{i},xs{i}] = ind2sub(size(segs),idx{i});
    
    if get_neighbors
        % crop around the segment to find neighbors
        miny = max(min(ys{i})-r-1,1); maxy = min(max(ys{i})+r+1,size(segs,1));
        minx = max(min(xs{i})-r-1,1); maxx = min(max(xs{i})+r+1,size(segs,2));

        % dilate the segment r pixels, and subtract itself to find touching
        % neighboring segment (and r pixels into them)
        touchingSP_mask = logical(imdilate(mask(miny:maxy,minx:maxx),s)-mask(miny:maxy,minx:maxx));
        neighboring_segs_edges = segs(miny:maxy,minx:maxx).*uint32(touchingSP_mask);

        % get all unique neighbors 
        neighbors{i} = unique(neighboring_segs_edges(neighboring_segs_edges>0)); neighbors{i}=neighbors{i}(:)';

        if pixel_neighbors
            % for each neighbor, get touching pixels, r pixels deep into the edge
            for j=neighbors{i}
                % find pixel indices within cropped image
                [ny,nx] = ind2sub([maxy-miny+1,maxx-minx+1],find(neighboring_segs_edges==j));
                % translate back into original image coordinates
                neighbors_edge_idx{i,j} = sub2ind(size(segs),miny+ny-1,minx+nx-1);

                neighbors_edge_xs{i,j} = minx-1+nx;
                neighbors_edge_ys{i,j} = miny-1+ny;
            end
        end
    else
        neighbors = {};
    end
end

sp_data = [];
sp_data.spidx = idx;
sp_data.xs = xs;
sp_data.ys = ys;
sp_data.neighbors = neighbors;
if pixel_neighbors
    sp_data.neighbors_edge_idx = neighbors_edge_idx;
    sp_data.neighbors_edge_xs = neighbors_edge_xs;
    sp_data.neighbors_edge_ys = neighbors_edge_ys;
end
    
