function generate_MIT_ROI_for_dnn(opts)
globals;
DATA_DIR = opts.paths.DATA_DIR;

load([DATA_DIR 'MIT/data.mat']);

% should use train or test
set_str = 'train';

img_ids = [];
M = uint8([]);
C = uint8([]);
P1 = uint8([]);
P2 = uint8([]);
BB = uint8([]);

fprintf('generating %s \n',set_str);

closer_field = opts.MIT_gen.closer_field;
closer_score_field = opts.MIT_gen.closer_score_field;

data_set = opts.MIT_gen.data_set;
%%
num_models = length(masks);
%%
for i=1:num_models
    fprintf('processing %d\n',i);
    for l=1:length(lights_images{i})
        %%
        tic
        

        I = im2uint8(lights_images{i}{l});
    
        % load precached json files
        load(sprintf([DATA_DIR 'MIT/%s%d_%d_comps.mat'],opts.MIT_gen.annotations_dir,i,l),'comps','points');

        if strcmp(closer_field,'closer') || strcmp(closer_field,'darker')
            fprintf('removing lots of equality edges\n');
            % randomly sample equality edges so we get roughly 60% equality
            eq_inds = find([comps.(closer_field)]=='E');
            eq_inds = eq_inds(randperm(length(eq_inds)));

            % get number of inequailty edges, if there are non skip image entirely
            num_ie = (length(comps)-length(eq_inds));
            if num_ie==0
               continue
            end

            % prune most of the equality edges, but leave 2 times as many equalities as inequialities
            eq_inds = eq_inds((2*num_ie):end);
            comps(eq_inds) = [];
        end

        % remove edges with closer score < 0
        comps([comps.(closer_score_field)]<0) = [];

        % now randomaly sample comps to use specified number
        comps_per_image = opts.MIT_gen.comps_per_image;
        if comps_per_image>0 && comps_per_image<length(comps)
            sub_inds = randperm(length(comps));
            sub_inds = sub_inds(comps_per_image+1:end);
            comps(sub_inds) = [];
        end
        %% extract keypoints

        tic
        [tP1,tP2,tBB,tM] = extract_patches_roi_from_comps(I,points,comps,opts);

        % get labels
        closer = [comps.(closer_field)];
        tC = zeros(1,length(closer)); tC(closer=='E')=0; tC(closer=='1')=1; tC(closer=='2')=2;

        P1(:,:,:,end+1:end+size(tP1,4)) = tP1;
        P2(:,:,:,end+1:end+size(tP2,4)) = tP2;
        M(:,:,:,end+1:end+size(tM,4)) = tM;
        BB(:,:,:,end+1:end+size(tBB,4)) = tBB;
        C(:,end+1:end+length(tC)) = tC;

        % collect image indices (output this later separately)
        img_ids(end+1:end+length(tC)) = repmat((i-1)*10+l,[1, length(tC)]);

        toc
    end
end
% fix weird end+1 behaviour, but not for C for some reason
M = M(:,:,:,2:end);
P1 = P1(:,:,:,2:end);
P2 = P2(:,:,:,2:end);
BB = BB(:,:,:,2:end);

%% randomly permute
rp = randperm(size(M,4));
M = M(:,:,:,rp);
P1 = P1(:,:,:,rp);
P2 = P2(:,:,:,rp);
BB = BB(:,:,:,rp);
C = C(rp);
img_ids = img_ids(rp);

%%
create_cmd_M = convert_to_lmdb(M(:,:,:,:),C,'M',data_set,set_str);
%%
create_cmd_P1 = convert_to_lmdb(P1(:,:,:,:),C,'P1',data_set,set_str);
%%
create_cmd_P2 = convert_to_lmdb(P2(:,:,:,:),C,'P2',data_set,set_str);
%%
create_cmd_BB = convert_to_lmdb(BB(:,:,:,:),C,'BB',data_set,set_str);
%% create list file and images for whole images
I_size = opts.params_ROI.I_size;
FIELD_NAME = 'I';
DATASET = data_set;
DB_NAME = set_str;
DB_DATA_DIR = fullfile(DATA_DIR,DATASET,DB_NAME);
LISTFILE_I = fullfile(DB_DATA_DIR,FIELD_NAME,'listfile.txt');
if (~exist(DB_DATA_DIR,'dir'))
    mkdir(DB_DATA_DIR);
end
if ~exist(fullfile(DB_DATA_DIR,FIELD_NAME),'dir')
    mkdir(fullfile(DB_DATA_DIR,FIELD_NAME));
end

fid = fopen(LISTFILE_I,'w');
for i=1:length(img_ids)
    curr_I_name = fullfile(DB_DATA_DIR,FIELD_NAME,sprintf('%d.jpg',img_ids(i)));
    fprintf(fid,[curr_I_name ' ' num2str(C(i)) '\n']);
end
fclose(fid);

unique_ids = unique(img_ids);
for i=unique_ids
    II = imresize(im2uint8(lights_images{ceil(i/10)}{(mod(i,10)+1)}),[I_size, I_size]);
    curr_I_name = fullfile(DB_DATA_DIR,FIELD_NAME,sprintf('%d.jpg',i));
    imwrite(II,curr_I_name);
end

create_cmd_I = strrep(create_cmd_M,'_M','_I');
create_cmd_I = strrep(create_cmd_I,'/M/','/I/');

%%
delete_I = sprintf('find %s%s%s/%s/ -name "*.jpg" -print0 | xargs -0 rm',DATA_DIR,[data_set '/'],set_str,'I');
delete_M = sprintf('find %s%s%s/%s/ -name "*.jpg" -print0 | xargs -0 rm',DATA_DIR,[data_set '/'],set_str,'M');
delete_P1 = sprintf('find %s%s%s/%s/ -name "*.jpg" -print0 | xargs -0 rm',DATA_DIR,[data_set '/'],set_str,'P1');
delete_P2 = sprintf('find %s%s%s/%s/ -name "*.jpg" -print0 | xargs -0 rm',DATA_DIR,[data_set '/'],set_str,'P2');
delete_BB = sprintf('find %s%s%s/%s/ -name "*.jpg" -print0 | xargs -0 rm',DATA_DIR,[data_set '/'],set_str,'BB');

%%
f = fopen('create_lmdb_nyu_roi.sh','w');
fprintf(f,'#!/usr/bin/env sh\n');
fprintf(f,[create_cmd_I '\n']);
fprintf(f,[create_cmd_M '\n']);
fprintf(f,[create_cmd_P1 '\n']);
fprintf(f,[create_cmd_P2 '\n']);
fprintf(f,[create_cmd_BB '\n']);
fprintf(f,[delete_I '\n']);
fprintf(f,[delete_M '\n']);
fprintf(f,[delete_P1 '\n']);
fprintf(f,[delete_P2 '\n']);
fprintf(f,[delete_BB '\n']);
fclose(f);
disp('dont forget to run create_lmdb_nyu_roi.sh');
