function generate_nyu_ms_for_dnn(opts)
globals;
DATA_DIR = opts.paths.DATA_DIR;

load([DATA_DIR 'NYU/nyu_depth_v2_labeled.mat']);
load([DATA_DIR 'NYU/splits_nyu.mat']);

%%
trainingidx = trainNdxs;
testidx = testNdxs;

% should use train or test
num_scales = opts.params_MS.num_scales;

if opts.NYU_gen.train
    train = 1;
else
    train = 0;
end
disp(train);

if train
    idx = trainingidx;
    set_str = 'train';
else
    idx = testidx;
    set_str = 'test';
end

fprintf('generating %s \n',set_str);

closer_field = opts.NYU_gen.closer_field;
closer_score_field = opts.NYU_gen.closer_score_field;

data_set = opts.NYU_gen.data_set;
%%
BB = cell(num_scales,1);
M = cell(num_scales,1);
for s=1:num_scales
    BB{s} = uint8([]);
    M{s} = uint8([]);
end

C = uint8([]);

for i=1:12;%length(idx)
    ind = idx(i);
    
    fprintf('processing %d\n',ind);
    
    %% load image
    I = im2uint8(images(:,:,:,ind));
    
    % load precached json files
    load(sprintf([DATA_DIR 'NYU/%s/%d_comps.mat'],opts.NYU_gen.annotations_dir,ind),'comps','points');

    if strcmp(closer_field,'closer')
        fprintf('removing lots of equality edges\n');
        % randomly sample equality edges so we get roughly 60% equality
        eq_inds = find([comps.(closer_field)]=='E');
        eq_inds = eq_inds(randperm(length(eq_inds)));
    
        % get number of inequailty edges, if there are non skip image entirely
        num_ie = (length(comps)-length(eq_inds));
        if num_ie==0
           continue
        end
    
        % prune most of the equality edges, but leave 2 times as many equalities as inequialities
        eq_inds = eq_inds((2*num_ie):end);
        comps(eq_inds) = [];
    end
    
    % remove edges with closer score < 0
    comps([comps.(closer_score_field)]<0) = [];
    
    % now randomaly sample comps to use specified number
    comps_per_image = opts.NYU_gen.comps_per_image;
    if comps_per_image>0 && comps_per_image<length(comps)
        sub_inds = randperm(length(comps));
        sub_inds = sub_inds(comps_per_image+1:end);
        comps(sub_inds) = [];
    end
    %% extract keypoints

    tic
    [tBB,tM] = extract_bb_ms_from_comps(I,points,comps,opts);
    
    % get labels
    closer = [comps.(closer_field)];
    tC = zeros(1,length(closer)); tC(closer=='E')=0; tC(closer=='1')=1; tC(closer=='2')=2;
    
    C(:,end+1:end+length(tC)) = tC;
    for s=1:num_scales
        M{s}(:,:,:,end+1:end+size(tM{s},4)) = tM{s};
        BB{s}(:,:,:,end+1:end+size(tBB{s},4)) = tBB{s};
    end
    toc
end
for s=1:num_scales% fix weird end+1 behaviour, but not for C for some reason
    M{s} = M{s}(:,:,:,2:end);
    BB{s} = BB{s}(:,:,:,2:end);
end

%% randomly permute
rp = randperm(size(M{1},4));
C = C(rp);
for s=1:num_scales
    M{s} = M{s}(:,:,:,rp);
    BB{s} = BB{s}(:,:,:,rp);
end
%% save to disk
dest_dir = [opts.paths.DATA_DIR opts.NYU_gen.data_set '/' set_str '/'];
if ~exist(dest_dir,'dir')
    mkdir(dest_dir);
end

save([dest_dir 'C.mat'],'C','-v7.3');
save([dest_dir 'M.mat'],'M','-v7.3');
save([dest_dir 'BB.mat'],'BB','-v7.3');

