function [R,S,points,comps] = do_intrinsic_decomposition(I,opts)


%% extract points, pairs and superpixel data
kp_params = opts.kp_params;

%% get second order neighbors for comparison, first order for smoothness
if opts.kp_params.use_multiscale
    % finest scale points
    [points,sp_data] = extract_key_points(I,kp_params);
    [comps,~] = get_point_pairs_ms(I,kp_params,points,sp_data);
else
    [points,sp_data] = extract_key_points(I,kp_params);
    [comps,~] = get_point_pairs(sp_data,true);
end
[~,ij] = get_point_pairs(sp_data,false);

%% get dnn estimates
[preds,fs] = infer_ordinal_relations_ROI_fast(I,points,comps,opts);
for t=1:length(preds)
    if (preds(t)==0)
        comps(t).darker = 'E';
    elseif (preds(t)==1)
        comps(t).darker = '1';
    else
        comps(t).darker = '2';
    end
    comps(t).pred_prob = fs(:,t);
    comps(t).darker_score = max(comps(t).pred_prob);
end

%% build smoothness term
I = im2double(I); I(I<=0) = 1/255; I(I>=1) = 1;

% get log mean luminance at each super pixel
npoints = length(points);
L = zeros(npoints,1);
t = mean(I,3);
for k=1:npoints
    L(k) = log(mean(t(sp_data.spidx{k})));
end
 
Lij = double(L(ij(:,1))-L(ij(:,2)));

ij = double(ij);

% build shading smoothness term
num_pairs = size(ij,1);
pair_inds = 1:num_pairs;
Ap = sparse([pair_inds(:); pair_inds(:)],[ij(:,1);ij(:,2)],[ones(num_pairs,1); -1*ones(num_pairs,1)],num_pairs,npoints);
% W = speye(num_pairs);
warning('still using image dependent smoothness for shading, better check this through');
lambda = 0.03;
W = sparse(1:num_pairs,1:num_pairs,exp(-(0.5/lambda^2)*Lij.^2),num_pairs,num_pairs);
A_prior = opts.ls_ranking_params.prior_weight*Ap'*W*Ap;
b_prior = accumarray([ij(:,1); ij(:,2)],opts.ls_ranking_params.prior_weight*[Lij; -Lij],[npoints 1]);
% b_prior = zeros(npoints,1);
Rs = do_ls_ranking(A_prior,b_prior,points,comps,opts);

% flood fill super pixels with estimated reflectance and shading
Rs = exp(Rs);
R = zeros(size(I,1),size(I,2));

% flood fill super pixels
S = zeros(size(R));
for i=1:npoints
    R(sp_data.spidx{i}) = Rs(i);
    S(sp_data.spidx{i}) = exp(L(i))./Rs(i);
end

if opts.intrinsic.blur_output
    R = imfilter(R,fspecial('gaussian',[5 5],1));
end
