function [f] = compute_DNN_features_generic(data_in,opts)


model_def_file = opts.DNN.model_def_file;
model_file = opts.DNN.model_file;
which_layer = opts.DNN.which_layer;
Nbatch = opts.DNN.Nbatch;

%%
assert(exist(model_def_file,'file')&&exist(model_file,'file'));
caffe('init', model_def_file, model_file);
%%


caffe_init_machine_specific;


caffe('set_phase_test');

%% preapre and get number of samples
data_in = prepare_DNN_input_generic(data_in);
num_samples = size(data_in{1},4);

% run the first batch just to get the feature size - TODO: find better way
caffe('forward',get_batch_data(data_in,1:Nbatch));
[features,names]=caffe('get_all_layers');
which_layer_idx = find(strcmp(names,which_layer));
if (isempty(which_layer_idx))
    error('no layer ''%s'' found',which_layer);
end

% actual size of the features
Nf_size = size(features{which_layer_idx});

%% compute features
f = zeros([Nf_size(1:3),num_samples]);

% run through the batches
for i=1:floor(num_samples/Nbatch)
    fprintf('processing batch %d\n',i);

    range = ((i-1)*Nbatch+1):(i*Nbatch);
    caffe('forward', get_batch_data(data_in,range));
    features=caffe('get_all_layers');

    f(:,:,:,range) = (features{which_layer_idx});
end

% left over bit
i=floor(num_samples/Nbatch);
range = ((i)*Nbatch+1):(i+1)*Nbatch;

caffe('forward', get_batch_data(data_in,range));
features=caffe('get_all_layers');
% get only the first part of the last batch
range = ((i)*Nbatch+1):num_samples;
f(:,:,:,range) = (features{which_layer_idx}(:,:,:,1:length(range)));

f = squeeze(f);

function batch_data = get_batch_data(data_in,batch_inds)
batch_data = cell(length(data_in),1);

% remove indices outside the dataset
pad_with_zeros = find(batch_inds>size(data_in{1},4));
batch_inds(pad_with_zeros) = [];
for i=1:length(data_in)
    batch_data{i} = data_in{i}(:,:,:,batch_inds);
    
    % pad with zeros if indices overflow
    if ~isempty(pad_with_zeros)
        batch_data{i} = cat(4,batch_data{i},zeros(size(data_in{i},1),size(data_in{i},2),size(data_in{i},3),length(pad_with_zeros)));
    end
end

