function [BB,M] = extract_bb_ms_from_comps(I,points,comps,opts)
%%


params = opts.params_MS;

num_scales = params.num_scales;

N = params.N; % half patch size
[xx,yy] = meshgrid(linspace(0,1,params.mask_size),linspace(0,1,params.mask_size));
blob_std = params.blob_std;

BB_size = params.BB_size;

BB = cell(num_scales,1);
M = cell(num_scales,1);
for s = 1:num_scales
    M{s} = uint8([]);
    BB{s} = uint8([]);
end

tic
% create image at all scales
I_s = cell(num_scales,1);
I_pad = cell(num_scales,1);
pad_h = cell(num_scales,1);
pad_w = cell(num_scales,1);
% I = im2double(I);
for s=1:num_scales
    I_s{s} = imresize(I,2^(-(s-1)),'bilinear');
    
    % get largest padding necessary and then some
    pad_h{s} = size(I_s{s},1);
    pad_w{s} = size(I_s{s},2);

    I_pad{s} = padarray(I_s{s},[pad_h{s} pad_w{s}],'both');
end

% get all rectangles at all scales around each point

rects = cell(num_scales,1);
is = cell(num_scales,1);
js = cell(num_scales,1);

npoints = length(points);
for s=1:num_scales
    is{s} = round([points.y]*size(I_s{s},1));
    js{s} = round([points.x]*size(I_s{s},2));
    
    P{s} = zeros(N*2+1,N*2+1,size(I,3),npoints);
    rects{s} = zeros(npoints,4);
    for p=1:npoints
        i = is{s}(p);
        j = js{s}(p);
        
        rects{s}(p,:) = [i-N,j-N,i+N,j+N];
    end
end


% generate masks and bb for each pair
for t=1:length(comps)
    for s=1:num_scales
        c = comps(t);

        p1 = c.point1;
        p2 = c.point2;

        i1 = is{s}(p1);
        j1 = js{s}(p1);

        i2 = is{s}(p2);
        j2 = js{s}(p2);

        % get rect for first patch [top, left, bottom, right]
        rect1 = rects{s}(p1,:);
        rect2 = rects{s}(p2,:);

        % extract bounding box of two patches
        bb_rect = [min(rect1(1:2),rect2(1:2))-N/4, max(rect1(3:4),rect2(3:4))+N/4];

        % make sure it's square (while centered)
        bb_w = bb_rect(4)-bb_rect(2);
        bb_h = bb_rect(3)-bb_rect(1);

        if bb_h>bb_w
            bb_rect(2) = bb_rect(2)-floor((bb_h-bb_w)/2);
            bb_rect(4) = bb_rect(4)+ceil((bb_h-bb_w)/2);
        else
            bb_rect(1) = bb_rect(1)-floor((bb_w-bb_h)/2);
            bb_rect(3) = bb_rect(3)+ceil((bb_w-bb_h)/2);
        end

        % now crop center to be desired size
        bb_cx = floor((bb_rect(4)+bb_rect(2))/2);
        bb_cy = floor((bb_rect(3)+bb_rect(1))/2);
        
        bb_rect(1) = bb_cy - BB_size/2;
        bb_rect(3) = bb_cy + BB_size/2;
        bb_rect(2) = bb_cx - BB_size/2;
        bb_rect(4) = bb_cx + BB_size/2;
        
        % now shift to snap to the image boundaries
        shift_top = max(-bb_rect(1)+1,0);
        shift_left = max(-bb_rect(2)+1,0);
        bb_rect([1, 3]) = bb_rect([1, 3]) + shift_top;
        bb_rect([2, 4]) = bb_rect([2, 4]) + shift_left;
        
        shift_bottom = min(size(I_s{s},1)-bb_rect(3),0);
        shift_right = min(size(I_s{s},2)-bb_rect(4),0);
        
        bb_rect([1, 3]) = bb_rect([1, 3]) + shift_bottom;
        bb_rect([2, 4]) = bb_rect([2, 4]) + shift_right;
        
        I_BB = I_pad{s}(pad_h{s}+(1+bb_rect(1):bb_rect(3)),pad_w{s}+(1+bb_rect(2):bb_rect(4)),:);
        
        % now build mask
        cy1 = (i1-bb_rect(1))/BB_size;
        cx1 = (j1-bb_rect(2))/BB_size;
        mask1 = exp(-0.5/blob_std.^2*((yy-cy1).^2 + (xx-cx1).^2));
        cy2 = (i2-bb_rect(1))/BB_size;
        cx2 = (j2-bb_rect(2))/BB_size;
        mask2 = exp(-0.5/blob_std.^2*((yy-cy2).^2 + (xx-cx2).^2));

        M{s}(:,:,:,end+1) = cat(3,im2uint8(mask1),im2uint8(mask2),im2uint8(zeros(size(mask1))));
        BB{s}(:,:,:,end+1) = I_BB;
        
        if opts.viz
            subplot(num_scales+1,3,(s-1)*3+1); showim(I_BB);
            subplot(num_scales+1,3,(s-1)*3+2); showim(mask1,[0 1]);
            subplot(num_scales+1,3,(s-1)*3+3); showim(mask2,[0 1]);
        end

    end
    
    if opts.viz
        subplot(num_scales+1,4,num_scales*4 + (1:4)); showim(I); axis image;
        hold on;
        i1 = is{1}(p1);
        j1 = js{1}(p1);

        i2 = is{1}(p2);
        j2 = js{1}(p2);
        plot(j1,i1,'rO');
        plot(j2,i2,'gO');
        hold off;
        drawnow;
    end
end

toc

for s=1:num_scales
    M{s} = M{s}(:,:,:,2:end);
    BB{s} = BB{s}(:,:,:,2:end);
end